-- scope stuff
qualityoflife = qualityoflife or {};
local q = qualityoflife;

q.modules = q.modules or {};
q.modules["QoL_ClassicHeals"] = q.modules["QoL_ClassicHeals"] or {};
local c = q.modules["QoL_ClassicHeals"];

function c:Println(str)
	if str then
		q:Println(str, "ClassicHeals");
	end
end

function c:GetAffectedPartyMemberId(memberName)
	for playerId, playerName in ipairs(q:GetPartyMembers()) do
		if playerName == memberName then
			return playerId;
		end
	end
end

function c:GetAffectedRaidMemberId(memberName)
	local rm = q:GetRaidMembers();
	local raidId;

	for playerId, playerName in ipairs(rm[0]) do
		if playerName == memberName then
			raidId = playerId;
			break;
		end
	end

	for subGroupId, subGroup in pairs(rm) do
		if subGroupId > 0 then
			for playerInSubGroupId, playerName in ipairs(rm[subGroupId]) do
				if playerName == memberName then
					return raidId, subGroupId, playerInSubGroupId;
				end;
			end;
		end
	end

	if memberName == GetUnitName("player") then
		return 1, 1, 1; -- for grid, vuhdo etc.
	end
end

function c:GetCompactRaidFrameId(memberName)
	for n = 1, 40 do
		local compactRaidFrameName = _G["CompactRaidFrame" .. n .. "Name"];
		if compactRaidFrameName and compactRaidFrameName:GetText() == memberName then
			return n;
		end
	end
end

function c:GetRaidPulloutId(subGroupId)
	for n = 1, 8 do
		local raidPulloutName = _G["RaidPullout" .. n .. "Name"];
		if raidPulloutName then
			local pulloutText = raidPulloutName:GetText();
			local textLen = string.len(pulloutText);
			local pulloutGroupId = tonumber(string.sub(pulloutText, textLen, textLen + 1));
			if pulloutGroupId == subGroupId then
				return n;
			end
		end
	end
end

function c:GetTargetFrames(target)
	local result, partyMemberId, raidId, subGroupId, playerInSubGroupId = {}, c:GetAffectedPartyMemberId(target), c:GetAffectedRaidMemberId(target);

	----- default wow / elvUI -----
	if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.OPT_UIATTR_ENABLED] then
		result[c.PROFILE_DEFAULT] = {};

		-- player frames
		if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PLAYER][c.OPT_UIATTR_ENABLED] and target == GetUnitName("player") then
			result[c.PROFILE_DEFAULT][c.PROFILE_PLAYER] = {};

			-- default player frame
			if _G["PlayerFrame"] and _G["PlayerFrame"]:IsVisible() then
				table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_PLAYER], _G["PlayerFrame"]);
			end

			-- elvui player frame
			if _G["ElvUF_Player"] and _G["ElvUF_Player"]:IsVisible() then
				table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_PLAYER], _G["ElvUF_Player"]);
			end
		end

		-- target frames
		if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGET][c.OPT_UIATTR_ENABLED] and target == GetUnitName("target") then
			result[c.PROFILE_DEFAULT][c.PROFILE_TARGET] = {};

			-- default target frame
			if _G["TargetFrame"] and _G["TargetFrame"]:IsVisible() then
				table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_TARGET], _G["TargetFrame"]);
			end

			-- elvui target frame
			if _G["ElvUF_Target"] and _G["ElvUF_Target"]:IsVisible() then
				table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_TARGET], _G["ElvUF_Target"]);
			end
		end

		-- party frames
		if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PARTY][c.OPT_UIATTR_ENABLED] and partyMemberId then
			result[c.PROFILE_DEFAULT][c.PROFILE_PARTY] = {};

			-- default party frame
			local defaultPartyFrameName = "PartyMemberFrame" .. partyMemberId;
			if _G[defaultPartyFrameName] and _G[defaultPartyFrameName]:IsVisible() then
				table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_PARTY], _G[defaultPartyFrameName]);
			end

			-- default party frame
			local elvUiPartyFrameName = "ElvUF_PartyGroup1UnitButton" .. partyMemberId + 1;
			if _G[elvUiPartyFrameName] and _G[elvUiPartyFrameName]:IsVisible() then
				table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_PARTY], _G[elvUiPartyFrameName]);
			end
		end
	end

	-- raid frames
	if raidId and subGroupId and playerInSubGroupId and (QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_COMPACT][c.OPT_UIATTR_ENABLED] or QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT][c.OPT_UIATTR_ENABLED]) then
		if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_COMPACT][c.OPT_UIATTR_ENABLED] then
			result[c.PROFILE_DEFAULT][c.PROFILE_RAID_COMPACT] = {};

			-- default compactraidgroup / compactraidframe
			if _G["CompactRaidFrameContainer"]:IsVisible() then
				local compactRaidFrameName;
				if CompactRaidFrameManager_GetSetting("KeepGroupsTogether") then
					compactRaidFrameName = "CompactRaidGroup" .. subGroupId .. "Member" .. playerInSubGroupId
				else
					local compactRaidFrameId = c:GetCompactRaidFrameId(target);
					if compactRaidFrameId then
						compactRaidFrameName = "CompactRaidFrame" .. compactRaidFrameId;
					end
				end

				if compactRaidFrameName and _G[compactRaidFrameName] and _G[compactRaidFrameName]:IsVisible() then
					table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_RAID_COMPACT], _G[compactRaidFrameName]);
				end
			end
		end

		if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT][c.OPT_UIATTR_ENABLED] then
			result[c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT] = {};

			-- default raid pull out group
			local pulloutGroupId = c:GetRaidPulloutId(subGroupId);
			if pulloutGroupId then
				local pulloutRaidFrameName = "RaidPullout" .. pulloutGroupId .. "Button" .. playerInSubGroupId;
				if _G[pulloutRaidFrameName] and _G[pulloutRaidFrameName]:IsVisible() then						
					table.insert(result[c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT], _G[pulloutRaidFrameName]);
				end
			end
		end
	end

	----- addons -----

	-- luna
	if c:GetProfile(c.PROFILE_ADDON_LUNA) and QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.OPT_UIATTR_ENABLED] then
		result[c.PROFILE_ADDON_LUNA] = {};

		-- player frame
		if QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.PROFILE_PLAYER][c.OPT_UIATTR_ENABLED] and target == GetUnitName("player") then
			result[c.PROFILE_ADDON_LUNA][c.PROFILE_PLAYER] = {};

			if _G["LUFUnitplayer"] and _G["LUFUnitplayer"]:IsVisible() then
				table.insert(result[c.PROFILE_ADDON_LUNA][c.PROFILE_PLAYER], _G["LUFUnitplayer"].highFrame);
			end
		end

		-- target frame
		if QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.PROFILE_TARGET][c.OPT_UIATTR_ENABLED] and target == GetUnitName("target") then
			result[c.PROFILE_ADDON_LUNA][c.PROFILE_TARGET] = {};

			if _G["LUFUnittarget"] and _G["LUFUnittarget"]:IsVisible() then
				table.insert(result[c.PROFILE_ADDON_LUNA][c.PROFILE_TARGET], _G["LUFUnittarget"].highFrame);
			end
		end

		-- party frame
		if partyMemberId and QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.PROFILE_PARTY][c.OPT_UIATTR_ENABLED] then
			result[c.PROFILE_ADDON_LUNA][c.PROFILE_PARTY] = {};

			local lunaFrameName = "LUFHeaderpartyUnitButton" .. partyMemberId;
			if _G[lunaFrameName] and _G[lunaFrameName]:IsVisible() then
				table.insert(result[c.PROFILE_ADDON_LUNA][c.PROFILE_PARTY], _G[lunaFrameName]);
			end
		end

		-- raid
		if subGroupId and playerInSubGroupId and QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.PROFILE_RAID_COMPACT][c.OPT_UIATTR_ENABLED] then
			result[c.PROFILE_ADDON_LUNA][c.PROFILE_RAID_COMPACT] = {};

			local lunaFrameName = "LUFHeaderraid" .. subGroupId .. "UnitButton" .. playerInSubGroupId;
			if _G[lunaFrameName] and _G[lunaFrameName]:IsVisible() then
				table.insert(result[c.PROFILE_ADDON_LUNA][c.PROFILE_RAID_COMPACT], _G[lunaFrameName]);
			end
		end
	end

	-- grid2
	if c:GetProfile(c.PROFILE_ADDON_GRID2) and subGroupId and playerInSubGroupId and QOL_OPTIONS[c.name][c.PROFILE_ADDON_GRID2][c.OPT_UIATTR_ENABLED] and QOL_OPTIONS[c.name][c.PROFILE_ADDON_GRID2][c.PROFILE_RAID_COMPACT][c.OPT_UIATTR_ENABLED] then
		result[c.PROFILE_ADDON_GRID2] = {};
		result[c.PROFILE_ADDON_GRID2][c.PROFILE_RAID_COMPACT] = {};

		local gridFrameName = "Grid2LayoutHeader" .. subGroupId .. "UnitButton" .. playerInSubGroupId;
		if _G[gridFrameName] and _G[gridFrameName]:IsVisible() and _G[gridFrameName].health then
			table.insert(result[c.PROFILE_ADDON_GRID2][c.PROFILE_RAID_COMPACT], _G[gridFrameName].health);
		end
	end

	-- vuhdo
	if c:GetProfile(c.PROFILE_ADDON_VUHDO) and QOL_OPTIONS[c.name][c.PROFILE_ADDON_VUHDO][c.OPT_UIATTR_ENABLED] then
		result[c.PROFILE_ADDON_VUHDO] = {};

		-- target
		if QOL_OPTIONS[c.name][c.PROFILE_ADDON_VUHDO][c.PROFILE_TARGET][c.OPT_UIATTR_ENABLED] and UnitName("target") then
			result[c.PROFILE_ADDON_VUHDO][c.PROFILE_TARGET] = {};

			local vuhdoTargetFrameName = c:GetVuhdoFrame("target");
			if vuhdoTargetFrameName and _G[vuhdoTargetFrameName .. "BgBarIcBarHlBarIc"] and _G[vuhdoTargetFrameName .. "BgBarIcBarHlBarIc"]:IsVisible() then
				table.insert(result[c.PROFILE_ADDON_VUHDO][c.PROFILE_TARGET], _G[vuhdoTargetFrameName .. "BgBarIcBarHlBarIc"]);
			end
		end

		if target == GetUnitName("player") and QOL_OPTIONS[c.name][c.PROFILE_ADDON_VUHDO][c.PROFILE_PLAYER][c.OPT_UIATTR_ENABLED] then
			result[c.PROFILE_ADDON_VUHDO][c.PROFILE_PLAYER] = {};
			-- player frame
			local vuhdoRaidFrameName = c:GetVuhdoFrame("player");
			if vuhdoRaidFrameName and _G[vuhdoRaidFrameName .. "BgBarIcBarHlBarIc"] and _G[vuhdoRaidFrameName .. "BgBarIcBarHlBarIc"]:IsVisible() then
				table.insert(result[c.PROFILE_ADDON_VUHDO][c.PROFILE_PLAYER], _G[vuhdoRaidFrameName .. "BgBarIcBarHlBarIc"]);
			end
		elseif target ~= GetUnitName("player") and QOL_OPTIONS[c.name][c.PROFILE_ADDON_VUHDO][c.PROFILE_RAID_COMPACT][c.OPT_UIATTR_ENABLED] then
			result[c.PROFILE_ADDON_VUHDO][c.PROFILE_RAID_COMPACT] = {};

			-- raid frame
			local vuhdoTarget;
			if IsInRaid() and raidId then
				vuhdoTarget = "raid" .. raidId;
			elseif IsInGroup() and partyMemberId then
				vuhdoTarget = "party" .. partyMemberId;
			end
			if vuhdoTarget then
				local vuhdoRaidFrameName = c:GetVuhdoFrame(vuhdoTarget);
				if vuhdoRaidFrameName and _G[vuhdoRaidFrameName .. "BgBarIcBarHlBarIc"] and _G[vuhdoRaidFrameName .. "BgBarIcBarHlBarIc"]:IsVisible() then
					table.insert(result[c.PROFILE_ADDON_VUHDO][c.PROFILE_RAID_COMPACT], _G[vuhdoRaidFrameName .. "BgBarIcBarHlBarIc"]);
				end
			end
		end
	end

	return result;
end

function c:GetVuhdoFrame(target)
	if VUHDO_PANEL_UNITS then
		-- kind of a brute force attempt. but the vuhdo code is way too obfuscated ^^
		for groupId = 1, 8 do
			for playerId = 1, 5 do
				local frameName = "Vd" .. groupId .. "H" .. playerId;
				if _G[frameName] and _G[frameName].raidid and _G[frameName].raidid == target then
					return frameName;
				end
			end
		end
	end
end

function c:ShowResetDialog()
	local dialogName = c.name .. "_RESETDIALOG";
	StaticPopupDialogs[dialogName] = StaticPopupDialogs[dialogName] or {
		text = q:GetText("This will reset all ClassicHeals settings to defaults.\n\nAre you sure?"),
		button1 = q:GetText("Reset"),
		button2 = q:GetText("Cancel"),
		OnEscapePressed = function ()
			self:Hide();
		end,
		OnAccept = function()
			QOL_OPTIONS[c.name] = {};
			c:InitOptions();
			c:InitProfiles(true);
			c:SaveSettingsChanges();
			c:SetProfileValues();
		end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
		preferredIndex = 3
	};
	StaticPopup_Show(dialogName);
end