-- scope stuff
qualityoflife = qualityoflife or {};
local q = qualityoflife;

local margin = 10;
local labelYOffset = 23;
local checkboxYOffset = 30;
local dropdownYOffset = 30;
local sliderYOffset = 35;
local spacingYOffset = 10;
local buttonYOffset = 30;

local elementIndex, yOffset = {}, {};
local oldValues = {};
local cbShowCrf;

function q:InitOptions()
	QOL_OPTIONS = QOL_OPTIONS or {};
	QOL_OPTIONS[q.name] = QOL_OPTIONS[q.name] or {};

	q.OPT_SHOWCRF = "OPT_SHOWCRF";
	if QOL_OPTIONS[q.name][q.OPT_SHOWCRF] == nil then
		QOL_OPTIONS[q.name][q.OPT_SHOWCRF] = false;
	end

	if not q.optionsPanel then
		q.optionsPanel = CreateFrame("Frame", "QoLOptionsFrame_Core");
		q.optionsPanel.name = "Quality of Life";
		--q.optionsPanel.okay = function(self) q:SaveOptions(); end;
		q.optionsPanel.cancel = function(self) q:RevertChanges(); end;
		InterfaceOptions_AddCategory(q.optionsPanel);
		q.optionsPanel:SetScript("OnShow", function()
			q:ShowOptionsPanel();
		end);

		local S = q:GetElvUiSkinModule();
		if S then
			S:HandleFrame(q.optionsPanel);
		end
	end
	q:ShowOptionsPanel();
end

function q:ShowOptionsPanel()
	oldValues = q:Deepcopy(QOL_OPTIONS[q.name]);

	if not cbShowCrf then
		cbShowCrf = q:CreateOptionsCheckbox(q.optionsPanel, q:GetText("Show Compact Raid Frames"), function(self)
			QOL_OPTIONS[q.name][q.OPT_SHOWCRF] = self:GetChecked();
			q:SetCrfVisible(QOL_OPTIONS[q.name][q.OPT_SHOWCRF]);
		end);
	end
	cbShowCrf:SetChecked(QOL_OPTIONS[q.name][q.OPT_SHOWCRF]);
end

function q:CreateOptionsChildFrame(parent, frameMargin, width, height)
	elementIndex[parent:GetName()] = elementIndex[parent:GetName()] or 0;
	frameMargin = frameMargin or margin;
	width = width or (623 - frameMargin * 2);
	height = height or (568 + yOffset[parent:GetName()] - frameMargin);

	local element = CreateFrame("Frame", "QoLOptionElement_" .. parent:GetName() .. "_" .. elementIndex[parent:GetName()], parent);
	element:SetPoint("TOPLEFT", frameMargin, yOffset[parent:GetName()]);
	element:SetWidth(width);
	element:SetHeight(height);

	local S = q:GetElvUiSkinModule();
	if S then
		S:HandleFrame(element);
	end

	yOffset[parent:GetName()] = yOffset[parent:GetName()] - height;
	elementIndex[parent:GetName()] = elementIndex[parent:GetName()] + 1;
	return element;
end

function q:CreateOptionsLabel(parent, text, xOffset, inLine)
	xOffset = xOffset or 0;
	yOffset[parent:GetName()] = yOffset[parent:GetName()] or -margin;
	elementIndex[parent:GetName()] = elementIndex[parent:GetName()] or 0;

	local element = CreateFrame("Frame", "QoLOptionElement_" .. parent:GetName() .. "_" .. elementIndex[parent:GetName()], parent);
	element:SetPoint("TOPLEFT", margin, yOffset[parent:GetName()]);
	element:SetWidth(1);
	element:SetHeight(23);
	element.fontString = element:CreateFontString("QoLOptionElement_" .. parent:GetName() .. "_" .. elementIndex[parent:GetName()] .. "_FontString", "ARTWORK", "GameFontNormal");
	element.fontString:SetPoint("LEFT", margin + xOffset, 0);
	element.fontString:SetText(text);

	function element.SetText(self, text)
		self.fontString:SetText(text);
	end

	function element.SetXOffset(self, xOffset)
		local point, relativeTo, relativePoint, xOfs, yOfs = self.fontString:GetPoint();
		self.fontString:SetPoint(point, xOffset, yOfs);
	end

	function element.SetTextColor(self, red, green, blue, alpha)
		self.fontString:SetTextColor(red, green, blue, alpha);
	end

	local S = q:GetElvUiSkinModule();
	if S then
		S:HandleFrame(element);
	end

	if not inLine then
		yOffset[parent:GetName()] = yOffset[parent:GetName()] - labelYOffset;
	end
	elementIndex[parent:GetName()] = elementIndex[parent:GetName()] + 1;
	return element;
end

function q:CreateOptionsCheckbox(parent, text, func, xOffset, inLine)
	xOffset = xOffset or 0;
	yOffset[parent:GetName()] = yOffset[parent:GetName()] or -margin;
	elementIndex[parent:GetName()] = elementIndex[parent:GetName()] or 0;

	local element = CreateFrame("CheckButton", "QoLOptionElement_" .. parent:GetName() .. "_" .. elementIndex[parent:GetName()], parent, "ChatConfigCheckButtonTemplate");
	element:SetPoint("TOPLEFT", margin + xOffset + 5, yOffset[parent:GetName()] -2);
	element:SetScript("OnClick", func);
	element.fontString = element:CreateFontString("QoLOptionElement_" .. parent:GetName() .. "_" .. elementIndex[parent:GetName()] .. "_FontString", "ARTWORK", "GameFontNormal");
	element.fontString:SetPoint("LEFT", 30, 0);
	element.fontString:SetText(text);

	function element.SetText(self, text)
		self.fontString:SetText(text);
	end

	function element.SetTextColor(self, red, green, blue, alpha)
		self.fontString:SetTextColor(red, green, blue, alpha);
	end

	local S = q:GetElvUiSkinModule();
	if S then
		S:HandleCheckBox(element);
	end

	if not inLine then
		yOffset[parent:GetName()] = yOffset[parent:GetName()] - checkboxYOffset;
	end
	elementIndex[parent:GetName()] = elementIndex[parent:GetName()] + 1;
	return element;
end

function q:CreateOptionsDropdown(parent, initFunc, xOffset, width, inLine)
	xOffset = xOffset or 0;
	width = width or 160;
	yOffset[parent:GetName()] = yOffset[parent:GetName()] or -margin;
	elementIndex[parent:GetName()] = elementIndex[parent:GetName()] or 0;

	local element = CreateFrame("Frame", "QoLOptionElement_" .. parent:GetName() .. "_" .. elementIndex[parent:GetName()], parent, "UIDropDownMenuTemplate");
	element:SetPoint("TOPLEFT", margin + xOffset - 10, yOffset[parent:GetName()]);
	UIDropDownMenu_SetWidth(element, width);
	UIDropDownMenu_Initialize(element, initFunc);

	function element.SetText(self, text)
		UIDropDownMenu_SetText(self, text);
	end

	local S = q:GetElvUiSkinModule();
	if S then
		S:HandleDropDownBox(element);
		element:ClearAllPoints();
		element:SetPoint("TOPLEFT", margin + xOffset - 11, yOffset[parent:GetName()]);
		UIDropDownMenu_SetWidth(element, width - 10);
	end

	if not inLine then
		yOffset[parent:GetName()] = yOffset[parent:GetName()] - dropdownYOffset;
	end
	elementIndex[parent:GetName()] = elementIndex[parent:GetName()] + 1;
	return element;
end

function q:CreateOptionsSlider(parent, min, max, func, xOffset, inLine, width, valueStep, minText, maxText)
	xOffset = xOffset or 0;
	min = min or 0;
	max = max or 100;
	valueStep = valueStep or 1;
	minText = minText or min;
	maxText = maxText or max;
	width = width or 160;
	yOffset[parent:GetName()] = yOffset[parent:GetName()] or -margin;
	elementIndex[parent:GetName()] = elementIndex[parent:GetName()] or 0;

	local element = CreateFrame("Slider", "QoLOptionElement_" .. parent:GetName() .. "_" .. elementIndex[parent:GetName()], parent, "OptionsSliderTemplate");
	element:SetPoint("TOPLEFT", margin + xOffset + 10, yOffset[parent:GetName()]);
	element:SetOrientation("HORIZONTAL");
	element:SetWidth(width);
	element:SetHeight(23);
	element:SetScript("OnValueChanged", func);
	element:SetMinMaxValues(min, max);
	element:SetValueStep(valueStep);
	element:SetObeyStepOnDrag(true);

	getglobal(element:GetName() .. "Low"):SetText(minText);
	getglobal(element:GetName() .. "High"):SetText(maxText);

	local S = q:GetElvUiSkinModule();
	if S then
		S:HandleSliderFrame(element);
	end

	if not inLine then
		yOffset[parent:GetName()] = yOffset[parent:GetName()] - sliderYOffset;
	end
	elementIndex[parent:GetName()] = elementIndex[parent:GetName()] + 1;
	return element;
end

function q:CreateOptionsButton(parent, text, func, xOffset)
	xOffset = xOffset or 0;
	yOffset[parent:GetName()] = yOffset[parent:GetName()] or -margin;
	elementIndex[parent:GetName()] = elementIndex[parent:GetName()] or 0;

	local element = CreateFrame("Button", "QoLOptionElement_" .. parent:GetName() .. "_" .. elementIndex[parent:GetName()], parent, "UIPanelButtonTemplate");
	element:SetPoint("TOPLEFT", 5 + margin + xOffset, -margin + yOffset[parent:GetName()]);
	element:SetHeight(23);
	element:SetWidth(180);
	element:SetScript("OnClick", func);
	element.fontString = element:CreateFontString("QoLOptionElement_" .. parent:GetName() .. "_" .. elementIndex[parent:GetName()] .. "_FontString", "ARTWORK", "GameFontNormal");
	element.fontString:SetPoint("CENTER");
	element.fontString:SetText(text);

	function element.SetText(self, text)
		self.fontString:SetText(text);
	end

	function element.SetTextColor(self, red, green, blue, alpha)
		self.fontString:SetTextColor(red, green, blue, alpha);
	end

	local S = q:GetElvUiSkinModule();
	if S then
		S:HandleButton(element);
	end

	yOffset[parent:GetName()] = yOffset[parent:GetName()] - dropdownYOffset;
	elementIndex[parent:GetName()] = elementIndex[parent:GetName()] + 1;
	return element;
end

function q:CreateOptionsSpacing(parent)
	yOffset[parent:GetName()] = yOffset[parent:GetName()] or -margin;
	yOffset[parent:GetName()] = yOffset[parent:GetName()] - spacingYOffset;
	return {};
end

function q:RevertChanges()
	QOL_OPTIONS[q:GetName()] = oldValues;
end